package rabbit.cache;

// $Id: NCacheEntry.java,v 1.7 2005/08/01 12:43:45 robo Exp $

import java.io.*;
import java.util.Date;

/** 
 *  A cached object.
 */
public class NCacheEntry implements Externalizable {
    private static final long serialVersionUID = 20050430;

    /** @serial The key for the object usually a URL or a filename.*/
    private Object key = null;
    /** @serial The date this entry was cached. */
    private long cachetime = -1;
    /** @serial The date this entry expires.*/
    private long expires = Long.MAX_VALUE;
    /** @serial The number of bytes this object is.*/
    private long size = 0;
    /** @serial The unique id of the object.*/
    private long id = 0;
    /** @serial The hooked data of the cached object. */
    private Object datahook;

    /** Not to be used, for externalizable only. */
    public NCacheEntry () {
    }

    /** Create a new CacheEntry for given key and filename
     * @param key the key for the object.
     * @param id the identity of this entry
     */
    public NCacheEntry (Object key, long id) {
	this.key = key;
	this.id = id;
    }    
    
    /** Set the key were holding data for
     * @param key the key we have data for
     */
    protected void setKey (Object key) {
	this.key = key;
    }
    
    /** Get the key were holding data for
     * @return the keyobject
     */
    public Object getKey () {
	Object ret = key;
	if (ret instanceof FiledKey) {
	    ret = ((FiledKey)ret).getData ();
	}
	return ret;
    }   
    
    /** Get the date this object was cached.
     * @return a date.
     */
    public long getCacheTime () {
	return cachetime;
    }

    /** Set the date this object was cached.
     * @param date the date.
     */
    public void setCacheTime (Date date) {
	cachetime = date.getTime ();
    }

    /** Get the size of our file
     * @return the size of our data
     */
    public long getSize () {
	return size;
    }  
    /** Sets the size of our data
     * @param size the new Size
     */
    public void setSize (long size) {
	this.size = size;
    }

    /** Get the expiry-date of our file
     * @return the expiry date of our data
     */
    public long getExpires () {
	return expires;
    }  
    /** Sets the expirydate of our data
     * @param d the new expiry-date.
     */
    public void setExpires (long d) {
	this.expires = d;
    }
    
    /** Get the id of our entry.
     * @return the id of the entry.
     */
    public long getId () {
	return id;
    }

    /** Get the real data hook
     */
    protected Object getRealDataHook () {
	return datahook;
    }

    /** Get the hooked data.
     * @param cache the NCache this entry lives in. 
     * @return the the hooked data.
     */
    public Object getDataHook (NCache cache) {
	if (datahook instanceof FiledHook) {
	    FiledHook fh = (FiledHook)datahook;
	    return fh.getData (cache, this);
	} else {
	    return datahook;
	}
    }  
    /** Sets the data hook for this cache object.
     *  Since it is not always possible to make the key hold this...
     * @param o the new data.
     */
    public void setDataHook (Object o) {
	this.datahook = o;
    }

    /** Read the cache entry from the object input. 
     */
    public void readExternal (ObjectInput in) 
	throws IOException, ClassNotFoundException {
	key = in.readObject ();
	cachetime = in.readLong ();
	expires = in.readLong ();
	size = in.readLong ();
	id = in.readLong ();
	datahook = in.readObject ();	
    }

    /** Write the object to the object output. 
     */
    public void writeExternal (ObjectOutput out) throws IOException {
	out.writeObject (key);
	out.writeLong (cachetime);
	out.writeLong (expires);
	out.writeLong (size);
	out.writeLong (id);
	out.writeObject (datahook);
    }
}
