package rabbit.filter;

// $Id: LowresImageFilter.java,v 1.11 2005/08/02 20:50:21 robo Exp $

import java.util.List;
import rabbit.html.*;
import rabbit.http.HTTPHeader;
import rabbit.proxy.Connection;

/** This filter removes the &quot;<tt>lowsrc=some_image.gif</tt>&quot; attributes
 *  from the &lt;img> tags.
 */
public class LowresImageFilter extends HTMLFilter {

    // For the factory.
    public LowresImageFilter () {
    }

    /** Create a new LowresImageFilter for the given request, response pair.
     * @param con the Connection handling the request.
     * @param request the actual request made.
     * @param response the actual response being sent.
     */
    public LowresImageFilter (Connection con, HTTPHeader request, HTTPHeader response) {	
	super (con, request, response);
    }

    @Override public HTMLFilter newFilter (Connection con, 
					   HTTPHeader request, 
					   HTTPHeader response) {
	return new LowresImageFilter (con, request, response);
    }

    /** remove the lowres tags.
     * @param block the part of the html page we are filtering.
     */
    @Override public void filterHTML (HTMLBlock block) {
	List<Token> tokens = block.getTokens ();
	int tsize = tokens.size ();
	for (int i = 0; i < tsize; i++) {
	    Token t = tokens.get (i);
	    switch (t.getType ()) {
	    case Token.TAG:
		Tag tag = t.getTag ();
		if (tag.getTagType () == TagType.IMG) {
		    tag.removeAttribute ("lowsrc");
		}
		break;
	    }
	}	
    }
}
