package rabbit.filter;

// $Id: ProxyAuth.java,v 1.9 2005/08/02 20:50:21 robo Exp $

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URL;
import java.util.*;
import rabbit.http.HTTPHeader;
import rabbit.proxy.Connection;
import rabbit.util.Logger;
import rabbit.util.SProperties;

/** This is a filter that requires users to use proxy-authentication.
 */
public class ProxyAuth implements HTTPFilter {
    private Map<String, String> users;
    
    /** test if a socket/header combination is valid or return a new HTTPHeader.
     *  Check that the user has been authenticate..
     * @param socket the Socket that made the request.
     * @param header the actual request made.
     * @param con the Connection handling the request.
     * @return null if everything is fine or a HTTPHeader describing the error (like a 403).
     */
    public HTTPHeader doHTTPInFiltering (Socket socket, HTTPHeader header, Connection con) {
	if (con.getMeta ())
	    return null;
	String username = con.getUserName ();
	String pwd = con.getPassWord ();
	if (username == null || pwd == null)
	    return getError (con, header);
	String rpwd = users.get (username);
	if (rpwd == null || !rpwd.equals (pwd))
	    return getError (con, header);	
	return null;
    }

    private HTTPHeader getError (Connection con, HTTPHeader header) {
	try {
	    return con.getResponseHandler ().get407 ("internet", 
						     new URL (header.getRequestURI ()));
	} catch (MalformedURLException e) {
	    con.getProxy ().logError (Logger.WARN, "Bad url: " + e);
	    return con.getResponseHandler ().get407 ("internet", null);
	}
    }

    /** test if a socket/header combination is valid or return a new HTTPHeader.
     *  does nothing.
     * @param socket the Socket that made the request.
     * @param header the actual request made.
     * @param con the Connection handling the request.
     * @return null if everything is fine or a HTTPHeader describing the error (like a 403).
     */
    public HTTPHeader doHTTPOutFiltering (Socket socket, HTTPHeader header, Connection con) {
	return null;
    }    

    /** Setup this class with the given properties.
     * @param logger the Logger to output errors/warnings on.
     * @param properties the new configuration of this class.
     */
    public void setup (Logger logger, SProperties properties) {
	String userfile = properties.getProperty ("userfile", "conf/allowed");
	try {
	    users = HTTPBaseFilter.loadUsers (new FileReader (userfile));
	} catch (FileNotFoundException e) {
	    logger.logError (Logger.WARN, 
			     "could not load the users file: '" + userfile + "'. " + e);
	} catch (IOException e) {
	    logger.logError (Logger.WARN, 
			     "Error while loading the users file: '" + userfile + "'. " + e);	    
	}
    }
}
