package rabbit.handler;

// $Id: MultiPartHandler.java,v 1.4 2005/08/02 20:50:21 robo Exp $

import java.io.DataInputStream;
import java.io.IOException;
import java.util.StringTokenizer;

import rabbit.http.HTTPHeader;
import rabbit.io.HTTPInputStream;
import rabbit.io.MultiOutputStream;
import rabbit.proxy.Connection;
import rabbit.util.Logger;

/** This class handles multipart responses, this handler does not 
 *  filter the resource.
 */
public class MultiPartHandler extends BaseHandler {

    private String boundary = null;

    // For creating the factory.
    public MultiPartHandler () {	
    }

    /** Create a new BaseHansler for the given request.
     * @param con the Connection handling the request.
     * @param request the actual request made.
     * @param response the actual response.
     * @param contentstream the stream to read data from.
     * @param clientstream the stream to write data to.
     * @param maycache May we cache this request? 
     * @param mayfilter May we filter this request?
     * @param size the size of the data beeing handled.
     */
    public MultiPartHandler (Connection con, 
			     HTTPHeader request, 
			     HTTPHeader response, 
			     HTTPInputStream contentstream, 
			     MultiOutputStream clientstream, 
			     boolean maycache,
			     boolean mayfilter, 
			     long size) {
	super (con, request, response, contentstream, clientstream, 
	       false, false, size);
	con.setChunking (false);

	//Content-Type: multipart/byteranges; boundary=B-mmrokjxyjnwsfcefrvcg\r\n	
	String ct = response.getHeader ("Content-Type");
	StringTokenizer st = new StringTokenizer (ct, ";= ");
	while (st.hasMoreTokens ()) {
	    String t = st.nextToken ();
	    if (t.equals ("boundary") && st.hasMoreTokens ()) {
		boundary = st.nextToken ();
		break;
	    }
	}
    }

    @Override public Handler getNewInstance (Connection connection,
					     HTTPHeader header, 
					     HTTPHeader webheader, 
					     HTTPInputStream contentStream, 
					     MultiOutputStream out, 
					     boolean maycache, 
					     boolean mayfilter, 
					     long size) {
	MultiPartHandler mph = new MultiPartHandler (connection, header, webheader, 
						     contentStream, out, maycache, 
						     mayfilter, size);
	return mph;
    }

    protected void send () throws IOException {
	DataInputStream dis = (DataInputStream)contentstream;
	String line;	
	while ((line = dis.readLine ()) != null) {
	    clientstream.write (line.getBytes ());
	    clientstream.write ('\r');
	    clientstream.write ('\n');
	    if (line.startsWith ("--") && line.endsWith ("--")) {
		if (line.substring (2, line.length () - 2).equals (boundary))
		    break;   // stop reading, we should not forward any epilogue
	    }
	}
    }
}
