package rabbit.io;

// $Id: MaxSizeOutputStream.java,v 1.4 2005/08/02 20:50:21 robo Exp $

import java.io.IOException;
import java.io.OutputStream;

/** A class that doesnt create outputstreams of more than a specified
 *  amount of data, the rest is simply swallowed
 */
public class MaxSizeOutputStream extends OutputStream 
{
    private long maxSize = -1;
    private long current = 0;
    private OutputStream os = null;

    /** Creates a new MaxSizeOutputStream with a given limit and
     *	connects it to a given stream.
     */
    public MaxSizeOutputStream (OutputStream os, long l) {
	current = 0;
	maxSize = l;
	this.os = os;
    }    

    /** Write a byte to this stream.
     * @param b the byte to write.
     * @throws IOException if the underlying stream does. 
     */
    public void write(int b) throws IOException {
	if (current < maxSize) {
	    os.write (b);
	    current++;
	} else {
	    throw new MaximumSizeExcededException ("Trying to write to a full stream");
	}
    }
    
    /** Write a byte array to this stream.
     * @param b the byte array to write.
     * @throws IOException if the underlying stream does. 
     */
    public void write(byte[] b) throws IOException {
	if ((current + b.length) < maxSize) {
	    os.write (b);
	    current += b.length;
	} else {
	    throw new MaximumSizeExcededException ("Trying to write to a full stream");
	}
    }
    
    /** Write a byte array to this stream.
     * @param b the byte to write.
     * @param off the starting offset.
     * @param len the number of bytes to write.
     * @throws IOException if the underlying stream does. 
     */
    public void write(byte[] b, int off, int len) throws IOException {
	if ((current + len) < maxSize) {
	    os.write (b, off, len);
	    current += len;
	} else if (maxSize - len < current) {
	    len = (int)(maxSize - current);
	    os.write (b, off, len);
	    current += len;
	    throw new MaximumSizeExcededException ("Trying to write to a full stream");	    
	} else {
	    throw new MaximumSizeExcededException ("Trying to write to a full stream");
	}
    }
    
    /** Write any pending data.
     * @throws IOException if the underlying stream does. 
     */
    public void flush() throws IOException {
	os.flush ();
    }
    
    /** Close this stream.
     * @throws IOException if the underlying stream does. 
     */
    public void close() throws IOException {
	os.close ();
    }
}
