package rabbit.meta;

// $Id: CacheStatus.java,v 1.14 2005/08/02 00:06:38 robo Exp $

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;

import rabbit.cache.*;
import rabbit.http.*;
import rabbit.html.*;
import rabbit.io.*;
import rabbit.proxy.*;
import rabbit.proxy.Proxy;

/** Get that status of the cache.
 */
public class CacheStatus implements MetaHandler {
    private static final int NUMBER_OF_ENTRIES = 256;

    /** default constructor */
    public CacheStatus () {
    }

    /** handler of a MetaPage.
     * @param input the InputStream from the client.
     * @param output the OutputStream to the client.
     * @param header the request.
     * @param htab the supplied argument to the page (CGI-parameters).
     * @param con the Connection that is serving this metapage.
     */
    public void handle (InputStream input, 
			MultiOutputStream output, 
			HTTPHeader header, 
			Properties htab, 
			Connection con) {
	try {   
	    NCache cache = con.getProxy ().getCache ();
	    long cursizemb = cache.getCurrentSize ();
	    cursizemb /= (1024 * 1024);
	    long maxsizemb = cache.getMaxSize ();
	    maxsizemb /= (1024 * 1024);
	    long cachetimeh =  cache.getCacheTime ();
	    cachetimeh /= (1000 * 60 * 60);
	    output.writeHTTPHeader (con.getResponseHandler ().getHeader ());
	    StringBuilder res = 
		new StringBuilder (HTMLPage.getPageHeader (con, "CacheStatus") + 
				   "Cachedir   : " + cache.getCacheDir () + ".<br>\n" +
				   "#Cached files: " + cache.getNumberOfEntries () + ".<br>\n" + 
				   "current Size: " + cursizemb + " MB.<br>\n" +
				   "Max Size: " + maxsizemb + " MB.<br>\n" +
				   "Cachetime : " + cachetimeh + " hours.<br>\n");
	    res.append ("<br>Partial listing of contents in cache, select entryset:<br>\n");
	    long entries = cache.getNumberOfEntries ();
	    for (long i = 0; i < Math.ceil (entries / (double)NUMBER_OF_ENTRIES); i++) {
		if (i > 0)
		    res.append (", "); 
		long j = i * NUMBER_OF_ENTRIES;
		long k = Math.min (j + NUMBER_OF_ENTRIES, entries);
		res.append ("<a href=\"CacheStatus?start=" + j + "\">" + j + "-" + k + "</a>"); 
	    }
	    
	    res.append (HTMLPage.getTableHeader (100, 1) +
			HTMLPage.getTableTopicRow () + 					     
			"<th>URL</th><th>filename</th><th>size</th><th>expires</th></tr>\n");
	    
	    output.write (res.toString ().getBytes ());
	    long totalsize = 0;
	    int count = 0;
	    String s = htab.getProperty ("start");
	    int start = 0; 
	    if (s != null) {
		try {
		    start = Integer.parseInt (s);
		} catch (NumberFormatException ex) {
		    ex.printStackTrace ();
		}
	    }
	    DateFormat sdf = new SimpleDateFormat ("yyyyMMdd-HH:mm");
	    Date d = new Date ();
	    for (NCacheEntry lister : cache.getEntries ()) {
		if (--start >= 0)
		    continue;
		HTTPHeader lheader = (HTTPHeader)lister.getKey ();
		if (lheader == null)
		    continue;
		// reading the data hook will cause lots of file reading... 
		//HTTPHeader webheader = (HTTPHeader)lister.getDataHook (cache);
		String filev = lheader.getRequestURI ();
		if (filev.length () > 60)
		    filev = filev.substring (0,57) + "...";
		d.setTime (lister.getExpires ());
		output.write (("<tr><td><a href = \"" + lheader.getRequestURI () + 
			       "\" target = cacheview>" + filev + "</a></td>" + 
			       "<td>" + cache.getEntryName (lister) + "</td>" +
			       "<td>" + lister.getSize () + "</td>" +
			       "<td>" + sdf.format (d) + 
			       "</td></tr>\n").getBytes ());
		totalsize += lister.getSize ();
		count++;
	    }
	    output.write (("<tr><td><B>Total:</b></td><td>&nbsp;</td><td>&nbsp;</td><td>" + 
			   totalsize + "</td><td>&nbsp;</td></tr>\n").getBytes ());
	    output.write (("</table>\n</body></html>").getBytes ());
	} catch (IOException e) {
	    con.getProxy ().logError ("Couldnt write cachestatus");
	}
    }
}
