
package rabbit.meta;

// $Id: Status.java,v 1.12 2005/08/03 17:00:53 robo Exp $

import java.io.*;
import java.net.*;
import java.util.*;

import rabbit.html.*;
import rabbit.http.*;
import rabbit.io.*;
import rabbit.proxy.*;
import rabbit.proxy.Proxy;
import rabbit.util.*;

/** the Metapage that shows the status of the proxy.
 */
public class Status implements MetaHandler {
    /** default constructor */
    public Status () {
    }

    /** handler of a MetaPage, that is a special page the proxy supports (like the status page).
     * @param input the InputStream from the client.
     * @param output the OutputStream to the client.
     * @param header the http request header
     * @param htab the supplied argument to the page (CGI-parameters).
     * @param con the Connection that is serving the request.
     */
    public void handle (InputStream input, 
			MultiOutputStream output, 
			HTTPHeader header, 
			Properties htab, 
			Connection con) {
	Proxy proxy = con.getProxy ();
	try {      
	    List convec = proxy.getCurrentConections ();
	    ThreadPool.Usage usage = proxy.getThreadPoolUsage ();
	    output.writeHTTPHeader (con.getResponseHandler ().getHeader ());
	    String res = HTMLPage.getPageHeader (con, "Status") +
		"Version: " + proxy.VERSION + "<br>\n" +
		"Running on: " + proxy.getHost () + " <B>:</B> " + proxy.getPort () + "<br>\n" + 
		"Started at: " + proxy.getStartDate () + "<br>\n" +
		"Current time: " + new Date () + "<br>\n" +
		"Alive and kicking with " + convec.size () + " current " +
		"connections.<br>\n" +
		"current free connections: " + usage.getFreeSize () + 
		"<br>\ncurrently used connections: " + usage.getUsedSize () + "<br>\n" + 
		HTMLPage.getTableHeader (100, 1) +
		HTMLPage.getTableTopicRow () + 
		"<th width=\"20%\">InetAddress</th><th>Id</th><th width=\"50%\">Connection</th>" +
		"<th width=\"20%\">Status</th><th>Time(s)</th></tr>\n";
	    output.write (res.getBytes ());
	    long now = new Date ().getTime ();
	    for (int i = 0; i < convec.size (); i++) {
		Connection hth = (Connection)convec.get (i);
		String s = "<tr><td>" + hth.getSocket ().getInetAddress ().getHostAddress () + 
		    "</td><td><nobr>" + hth.getName () + 
		    "</nobr></td><td>"+ hth.getRequestLine () +
		    "</td><td>" + hth.getStatus () + 
		    "</td><td>" + ((now - hth.getStarted ().getTime())/1000) + "</td></tr>\n";
		output.write (s.getBytes ());
	    }
	
	    String s = "</table>\n<br>\n" + HTMLPage.getTableHeader (100, 1) + HTMLPage.getTableTopicRow ();
	    output.write (s.getBytes ());
	    s = "<th>thingy</th><th width=\"10%\">times</th></tr>\n";
	    output.write (s.getBytes ());
	
	    // add the logged things.
	    Set<String> e = proxy.getCounter ().keys ();
	    List<String> ls = new ArrayList<String> (e);
	    Collections.sort (ls);
	    for (String type : ls) {
		int val = proxy.getCounter ().get (type);
		s = "\t<tr><td>" + type + "</td><td>" + val + "</td></tr>\n";
		output.write (s.getBytes ());
	    }
	
	    output.write ("</table>\n</body></html>".getBytes ());
	} catch (IOException e) {
	    proxy.logError ("Couldnt write status");
	}
    }
}
