package rabbit.proxy;

// $Id: MultiProxy.java,v 1.4 2005/05/22 12:13:08 robo Exp $

import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.*;

/** An applications that can load multiple instances of Proxy, 
 *  each with a different configuration.
 */
public class MultiProxy {

    /** The ClassLoader used to separate the instances of Proxy.
     */
    public static class CustomClassLoader extends ClassLoader {
	
	private Map<String, Class> loadedClasses = new HashMap<String, Class> ();
	public Class loadClass (String name, boolean resolve) throws ClassNotFoundException {
	    // First, check if the class has already been loaded
	    Class c = findLoadedClass(name);	    
	    if (c == null) {
		try {
		    c = loadedClasses.get (name);
		    if (c == null) {
			c = findClass (name);
			loadedClasses.put (name, c);
		    }
		} catch (ClassNotFoundException e) {		    
		    //e.printStackTrace ();
		}
		if (c == null)
		    return super.loadClass (name, resolve);
	    }
	    if (resolve) {
		resolveClass(c);
	    }
	    return c;	    
	}

	protected Class findClass (String name) throws ClassNotFoundException {	    
	    try {
		String path = name.replace ('.', '/') + ".class";
		File f = new File (path);
		if (!f.exists ())
		    throw new ClassNotFoundException ("not found: " + name);
		DataInputStream fi = new DataInputStream (new FileInputStream (path));
		byte[] classBytes = new byte[(int)f.length ()];
		fi.readFully (classBytes);
		fi.close ();
		return defineClass(name, classBytes, 0, classBytes.length);
	    } catch (Exception e) {
		// We could not find the class, so indicate the problem with an exception
		throw new ClassNotFoundException (name);
	    }	    
	}
    }

    public static void main (String[] args) {
	for (int i = 0; i < args.length; i++) {
	    try {
		CustomClassLoader ucl = new CustomClassLoader ();
		Class pc = ucl.loadClass ("rabbit.proxy.Proxy");
		Object[] pargs = new Object[1];
		String[] sargs = new String[]{"-f", args[i]};
		pargs[0] = sargs;
		Class[] cargs = new Class[]{String[].class};
		Method m = pc.getDeclaredMethod ("main", cargs);
		m.invoke (pc, pargs);
	    } catch (Exception e) {
		e.printStackTrace ();
	    }
	}
    }
}
