package rabbit.proxy;

// $Id: SSLHandler.java,v 1.6 2005/08/02 20:50:21 robo Exp $

import java.net.SocketException;
import java.net.SocketTimeoutException;
import rabbit.http.HTTPHeader;
import rabbit.io.HTTPInputStream;
import rabbit.io.HTTPOutputStream;
import rabbit.io.MultiOutputStream;
import rabbit.io.WebConnection;
import rabbit.util.Coder;

/** SSL handling methods.
 */
class SSLHandler {
    
    private Proxy proxy;

    protected SSLHandler (Proxy proxy) {
	this.proxy = proxy;
    }

    /** Check to see if this header is an SSL header.
     * @param header the header to check.
     * @return true if this header is an SSL request.
     */
    public static boolean isSSLRequest (HTTPHeader header) {
	return header.getMethod ().equals ("CONNECT");
    }


    /** Are we allowed to proxy ssl-type connections ?
     * @return true if we allow the CONNECT &lt;port&gt; command.
     */	
    public boolean sslIsAllowed (HTTPHeader header) {
	String hp = header.getRequestURI ();
	int c = hp.indexOf (':');
	Integer port = new Integer (443);
	if (c >= 0) {
	    try {
		port = new Integer (hp.substring (c+1));
	    } catch (Exception e) {
		// strange request, dont allow it.
		return false;
	    }
	}
	if (proxy.proxySSL == false)
	    return false;
	if (proxy.proxySSL == true && proxy.sslports == null)
	    return true;
	for (int i = 0; i < proxy.sslports.size (); i++) {
	    if (port.equals (proxy.sslports.get (i)))
		return true;
	}
	return false;
    }
    
    /** Handle the SSL request
     * @param header the Connect request header.
     * @param in the client input stream
     * @param out the client output stream
     * @param connection the Connection handling the request
     */
    protected TunnelHandler.HandleMode handleSSL (HTTPHeader header, 
						  HTTPInputStream in, 
						  MultiOutputStream out, 
						  Connection connection) {
	String uri = header.getRequestURI ();
	header.setRequestURI ("http://" + uri);  // java needs proto to build URL
	WebConnection wc = null;
	TunnelHandler.HandleMode ret = TunnelHandler.HandleMode.THREADED;
	try {
	    if (proxy.isProxyConnected ()) {
		String auth = proxy.getProxyAuthString ();
		// it should look like this (using RabbIT:RabbIT):
		// Proxy-authorization: Basic UmFiYklUOlJhYmJJVA==
		header.setHeader ("Proxy-authorization", "Basic " + Coder.uuencode (auth));
	    }
	    
	    wc = proxy.getWebConnection (header);
	    if (proxy.isProxyConnected ()) {
		header.setRequestURI (uri); // send correct connect to next proxy.
		// verify that we can connect here...
		HTTPOutputStream os = wc.getOutputStream ();
		os.writeHTTPHeader (header, proxy.isProxyConnected (), 
				    proxy.getProxyAuthString ());
		HTTPInputStream is = wc.getInputStream ();
		HTTPHeader response = is.readHTTPHeader (); 
		String status = response.getStatusCode (); 
		if (!"200".equals (status))
		    return ret;
	    }
	    HTTPHeader reply = new HTTPHeader ();
	    reply.setStatusLine ("HTTP/1.0 200 Connection established");
	    reply.setHeader ("Proxy-agent", proxy.getServerIdentity ());
	    out.writeHTTPHeader (reply);	    
	    ret = proxy.getTunnelHandler ().tunnel (in, out, wc);
	} catch (SocketException e) {
	    // ignore since this will most probably be the closing of the connection...
	} catch (SocketTimeoutException e) {
	    // as above.
	} catch (Exception e) {
	    e.printStackTrace ();
	    connection.doError (400, e);
	} finally {
	    if (ret == TunnelHandler.HandleMode.THREADED && wc != null) {
		wc.close ();
	    }		
	}
	return ret;
    }
}
